@@ -4,27 +4,43 @@ Add various autogeneratable pages to the docs
 
 See the Flags (-h) regarding possibilities.
 """
+import importlib
+import inspect
+import json
 import os
+import shutil
+import sys
+import time
+import traceback
+from ast import literal_eval
+from contextlib import contextmanager
 from functools import partial
+from hashlib import md5
 
-
-import time, json
-from devapp.tools import FLG, project, gitcmd, parse_kw_str
-from devapp.app import app, run_app, system, do as app_do
-
-from pymdownx import slugs
-from theming.formatting import markdown
-from inflection import humanize
-from devapp.tools import deindent, cast
-from lcdoc.auto_docs import mark_auto_created
-
+import toml
+from devapp.app import app
+from devapp.app import do as app_do
+from devapp.app import run_app, system
 from devapp.testing.auto_docs import func_title
 
-
 # from operators.testing.auto_docs import dir_pytest_base
-from devapp.tools import project, read_file, write_file
-from hashlib import md5
-import shutil
+from devapp.tools import (
+    FLG,
+    cast,
+    deindent,
+    gitcmd,
+    parse_kw_str,
+    project,
+    read_file,
+    walk_dir,
+    write_file,
+)
+from inflection import humanize
+from pymdownx import slugs
+from theming.formatting import markdown
+
+from lcdoc import lp as lit_prog
+from lcdoc.auto_docs import mark_auto_created
 
 
 def do(action, *a, track=False, **kw):
@@ -60,6 +76,16 @@ class Flags:
         n = 'Create a changelog (using git-changelog)'
         d = False
 
+    class gen_change_log_commit_style:
+        n = 'argument for git-changelog'
+        d = 'angular'
+        t = ['angular', 'basic', 'atom']
+
+    class gen_change_log_versioning_stanza:
+        n = 'Header message in changelog'
+        d = 'calver'
+        t = ['calver', 'semver']
+
     class gen_mod_docs:
         n = 'Supply list of module files to insert into "# :automodocs:" block in mkdocs.yml'
         d = []
@@ -233,14 +259,10 @@ class Credits:
             direct_dependencies = {dep.lower() for dep in project.dependencies()}
             dev_dependencies = {dep.lower() for dep in project.dev_dependencies()}
             'python' in direct_dependencies and direct_dependencies.remove('python')
-            indirect_dependencies = {
-                pkg['name'].lower() for pkg in lock_data['package']
-            }
+            indirect_dependencies = {pkg['name'].lower() for pkg in lock_data['package']}
             indirect_dependencies -= direct_dependencies
             indirect_dependencies -= dev_dependencies
-            dependencies = (
-                direct_dependencies | dev_dependencies | indirect_dependencies
-            )
+            dependencies = direct_dependencies | dev_dependencies | indirect_dependencies
 
             packages = {}
             attrs = ('name', 'home-page', 'license', 'version', 'summary')
@@ -304,8 +326,6 @@ class Credits:
 
 Credits.TP = deindent(Credits.TP)
 Credits.TC = deindent(Credits.TC)
-import sys
-from devapp.tools import walk_dir
 
 d_autodocs = lambda: project.root() + '/docs/autodocs'
 fn_autodocs_refs = lambda: d_autodocs() + '/references.md'
@@ -317,9 +337,6 @@ def scan_d_autodocs(typ):
     return files
 
 
-from contextlib import contextmanager
-
-
 class SVGs:
     @contextmanager
     def has_changed(fn, ext):
@@ -397,9 +414,7 @@ class SVGs:
                     #    FLG.kroki_server + '/plantuml/svg/' + g.decode('utf-8')
                     # )
                     if not res.status_code < 300:
-                        raise Exception(
-                            'kroki_server: [%s] not [200]' % res.status_code
-                        )
+                        raise Exception('kroki_server: [%s] not [200]' % res.status_code)
                 except Exception as ex:
                     app.die(
                         'No svg from kroki', exc=ex, resp=getattr(res, 'text', 'n.a.')
@@ -619,27 +634,21 @@ def patch_mkdocs_filewatch_ign_lp():
     import mkdocs
 
     fn = mkdocs.__file__.rsplit('/', 1)[0]
-    fn += '/commands/serve.py'
-    s = read_file(fn).splitlines()
-    r = []
-    while s:
-        l = s.pop(0)
-        if 'server.watch' in l and 'docs_dir' in l:
-            if '**/*.md' in l:
-                return app.info('Already patched', fn=fn)
-            msg = '    # !! patched by docutools to just watch .md files - not all:'
-            r.append('\n')
-            r.append(msg.upper())
-            r.append("    server.watch(config['docs_dir'] + '/**/*.md', builder)")
-            r.append('\n')
+    fn += '/livereload/__init__.py'
 
-        else:
-            r.append(l)
-    write_file(fn, '\n'.join(r))
-    app.warn('Have patched to only watch .md files', fn=fn)
+    if not exists(fn):
+        return app.warn('Cannot patch mkdocs - version mismatch', missing=fn)
 
-
-import importlib, inspect
+    s = read_file(fn)
+    S = 'event.is_directory'
+    if not S in s:
+        return app.warn('Cannot patch mkdocs - version mismatch', missing=fn)
+    if not "'.lp'" in s:
+        new = S + " or event.src_path.endswith('.lp')"
+        s = s.replace(S, new)
+        write_file(fn, s)
+        return app.warn('Have patched to only watch .md files', fn=fn)
+    app.info('Already patched', fn=fn)
 
 
 def is_func(obj, fn, src):
@@ -733,10 +742,30 @@ class ModDocs:
 
 
 def gen_change_log():
-    dr, d = S.d_root, dirname
-    cmd = 'cd "%s"; git-changelog -t "path:%s" . > CHANGELOG.md'
+    """
+    Problem: The git-changelog cmd uses Jinja and wants .md sources.
+    mkdocs macros also, different contexts though -> crash.
+    So we cannot have .md sources in the docs folder -> have to change on the fly when
+    using git-changelog. Here we do that. Also we dyn set the versioning message.
+    """
+    dr, d, dtmp = S.d_root, dirname, S.d_root + '/tmp/changelog_tmpl'
+
+    def set_version_scheme(fn):
+        CV = 'This project adheres to [CalVer Versioning](http://calver.org) ![](https://img.shields.io/badge/calver-YYYY.M.D-22bfda.svg).'
+        SV = 'This project adheres to [Semantic Versioning](http://semver.org/spec/v2.0.0.html).'
+        VSM = CV if FLG.gen_change_log_versioning_stanza == 'calver' else SV
+        s = read_file(fn).replace('_VERSION_SCHEME_MSG_', VSM)
+        write_file(fn, s)
+
     dcl = d_assets() + '/mkdocs/lcd/src/md/keepachangelog'
-    cmd = cmd % (dr, dcl)
+    os.makedirs(dtmp, exist_ok=True)
+    for k in os.listdir(dcl):
+        fn = dtmp + '/' + k.replace('.tmpl', '')
+        shutil.copyfile(dcl + '/' + k, fn)
+        if k == 'changelog.md.tmpl':
+            set_version_scheme(fn)
+    cmd = 'cd "%s"; git-changelog -s %s -t "path:%s" . > CHANGELOG.md'
+    cmd = cmd % (dr, FLG.gen_change_log_commit_style, dtmp)
     err = os.system(cmd)
     if err:
         app.die('changelog creation failed', fn=dcl, cmd=cmd)
@@ -783,9 +812,6 @@ def gen_modify_date():
     write_file(fn, r)
 
 
-import toml
-
-
 def add_pyproject_infos_to_mkdocs():
     # TODO
     raise Exception('TODO: Convert to pdm!')
@@ -808,13 +834,6 @@ def add_pyproject_infos_to_mkdocs():
     write_file(fnm, m)
 
 
-from lcdoc import lp as lit_prog
-
-import traceback
-
-from ast import literal_eval
-
-
 class LP:
     """literate programming feature"""
 
@@ -840,8 +859,11 @@ class LP:
 
     run_file = lambda fn_lp: LP.run_md(read_file(fn_lp), fn_lp, write=True)
 
-    def run_md(md, fn_lp, write=False):
-        """fn_lp required for filenames of async lp results"""
+    def run_md(md, fn_lp, write=False, raise_on_errs=None):
+        """fn_lp required for filenames of async lp results
+        raise_on_errs intended for temporarily changing behviour, e.g. for tests
+        Else use the FLG.
+        """
         S.cur_fn_lp = fn_lp
         lp_blocks, dest = do(LP.extract_lp_blocks, md=md, fn_lp=fn_lp)
         app.info('---------- %s --------------' % fn_lp)
@@ -849,7 +871,10 @@ class LP:
         res = []
         # the doc file:
         fnd = LP.fn_lp(fn_lp)
-        [res.append(LP.run_block(block, fnd)) for block in lp_blocks]
+        [
+            res.append(LP.run_block(block, fnd, raise_on_errs=raise_on_errs))
+            for block in lp_blocks
+        ]
         md = '\n'.join(dest)
         for nr in range(len(res)):
             md = md.replace(LP.PH(nr), res[nr])
@@ -864,7 +889,7 @@ class LP:
             print('(Evaluated LP Blocks: %s)\n\n' % len(lp_blocks))
         return md
 
-    def run_block(block, fnd):
+    def run_block(block, fnd, raise_on_errs=None):
         cmd, kw = '', ''
         try:
             args, kw = block['args'], block['kwargs']
@@ -888,7 +913,9 @@ class LP:
             S.stats.count_lp_blocks += 1
             res = run_lp(cmd, *args, **kw)
         except Exception as e:
-
+            # intended for pytesting lp itself:
+            if raise_on_errs:
+                raise
             if not S.lp_on_err_keep_running:
                 app.die('Could not eval', exc=e)
             if LP.interrupted in str(e):
@@ -928,10 +955,15 @@ class LP:
             return (), parse_kw_str(H, header_kws, try_json=False)
         except Exception as ex:
             ex1 = ex
-            try:
-                return eval('get_args(%s)' % H, header_kws, {})
-            except Exception as ex:
-                ex2 = ex
+            # evaling now.
+            # still - we supply only a minimum eval ctx and prevent
+            # imports. But still this won't be totally safe.
+            # BUT: Hey - we are about to run code *anyway*, this is LP in the end!
+            if not 'import' in H:
+                try:
+                    return eval('get_args(%s)' % H, header_kws, {})
+                except Exception as ex:
+                    ex2 = ex
         return LP.header_parse_err, {LP.py_err: ex2, LP.easy_args_err: ex1, 'header': H}
 
     get_args = lambda *a, **kw: [a, kw]  # trick to get python sig args
